@@ -19,7 +19,6 @@ import sys
 from sqlalchemy import event
 from sqlalchemy import exc as sqla_exc
 
-from oslo_db._i18n import _LE
 from oslo_db import exception
 
 
@@ -62,31 +61,31 @@ def filters(dbname, exception_type, regex):
 @filters("mysql", sqla_exc.DatabaseError,
          r"^.*\b1205\b.*Lock wait timeout exceeded.*")
 @filters("mysql", sqla_exc.InternalError, r"^.*\b1213\b.*Deadlock found.*")
+@filters("mysql", sqla_exc.InternalError,
+         r"^.*\b1213\b.*detected deadlock/conflict.*")
+@filters("mysql", sqla_exc.InternalError,
+         r"^.*\b1213\b.*Deadlock: wsrep aborted.*")
 @filters("postgresql", sqla_exc.OperationalError, r"^.*deadlock detected.*")
 @filters("postgresql", sqla_exc.DBAPIError, r"^.*deadlock detected.*")
-@filters("ibm_db_sa", sqla_exc.DBAPIError, r"^.*SQL0911N.*")
 def _deadlock_error(operational_error, match, engine_name, is_disconnect):
     """Filter for MySQL or Postgresql deadlock error.
 
     NOTE(comstud): In current versions of DB backends, Deadlock violation
     messages follow the structure:
 
-    mysql+mysqldb:
-    (OperationalError) (1213, 'Deadlock found when trying to get lock; try '
-                         'restarting transaction') <query_str> <query_args>
+    mysql+mysqldb::
 
-    mysql+mysqlconnector:
-    (InternalError) 1213 (40001): Deadlock found when trying to get lock; try
-                         restarting transaction
+        (OperationalError) (1213, 'Deadlock found when trying to get lock; '
+            'try restarting transaction') <query_str> <query_args>
 
-    postgresql:
-    (TransactionRollbackError) deadlock detected <deadlock_details>
+    mysql+mysqlconnector::
 
+        (InternalError) 1213 (40001): Deadlock found when trying to get lock;
+            try restarting transaction
 
-    ibm_db_sa:
-    SQL0911N The current transaction has been rolled back because of a
-    deadlock or timeout <deadlock details>
+    postgresql::
 
+        (TransactionRollbackError) deadlock detected <deadlock_details>
     """
     raise exception.DBDeadlock(operational_error)
 
@@ -117,6 +116,12 @@ def _default_dupe_key_error(integrity_error, match, engine_name,
     N columns - (IntegrityError) duplicate key value violates unique
                constraint "name_of_our_constraint"
 
+    mysql since 8.0.19:
+    1 column - (IntegrityError) (1062, "Duplicate entry 'value_of_c1' for key
+               'table_name.c1'")
+    N columns - (IntegrityError) (1062, "Duplicate entry 'values joined
+               with -' for key 'table_name.name_of_our_constraint'")
+
     mysql+mysqldb:
     1 column - (IntegrityError) (1062, "Duplicate entry 'value_of_c1' for key
                'c1'")
@@ -128,9 +133,6 @@ def _default_dupe_key_error(integrity_error, match, engine_name,
                key 'c1'
     N columns - (IntegrityError) 1062 (23000): Duplicate entry 'values
                joined with -' for key 'name_of_our_constraint'
-
-
-
     """
 
     columns = match.group('columns')
@@ -142,6 +144,9 @@ def _default_dupe_key_error(integrity_error, match, engine_name,
     if not columns.startswith(uniqbase):
         if engine_name == "postgresql":
             columns = [columns[columns.index("_") + 1:columns.rindex("_")]]
+        elif (engine_name == "mysql") and \
+             (uniqbase in str(columns.split("0")[:1])):
+            columns = columns.split("0")[1:]
         else:
             columns = [columns]
     else:
@@ -172,7 +177,6 @@ def _sqlite_dupe_key_error(integrity_error, match, engine_name, is_disconnect):
 
     sqlite since 3.8.2:
     (IntegrityError) PRIMARY KEY must be unique
-
     """
     columns = []
     # NOTE(ochuprykov): We can get here by last filter in which there are no
@@ -192,15 +196,16 @@ def _sqlite_dupe_key_error(integrity_error, match, engine_name, is_disconnect):
          r"(?i).*foreign key constraint failed")
 @filters("postgresql", sqla_exc.IntegrityError,
          r".*on table \"(?P<table>[^\"]+)\" violates "
-         "foreign key constraint \"(?P<constraint>[^\"]+)\".*\n"
-         "DETAIL:  Key \((?P<key>.+)\)=\(.+\) "
-         "is (not present in|still referenced from) table "
-         "\"(?P<key_table>[^\"]+)\".")
-@filters("mysql", sqla_exc.IntegrityError,
-         r".*Cannot (add|delete) or update a (child|parent) row: "
-         'a foreign key constraint fails \([`"].+[`"]\.[`"](?P<table>.+)[`"], '
-         'CONSTRAINT [`"](?P<constraint>.+)[`"] FOREIGN KEY '
-         '\([`"](?P<key>.+)[`"]\) REFERENCES [`"](?P<key_table>.+)[`"] ')
+         r"foreign key constraint \"(?P<constraint>[^\"]+)\".*\n"
+         r"DETAIL:  Key \((?P<key>.+)\)=\(.+\) "
+         r"is (not present in|still referenced from) table "
+         r"\"(?P<key_table>[^\"]+)\".")
+@filters(
+    "mysql", sqla_exc.IntegrityError,
+    r".*Cannot (add|delete) or update a (child|parent) row: "
+    r'a foreign key constraint fails \([`"].+[`"]\.[`"](?P<table>.+)[`"], '
+    r'CONSTRAINT [`"](?P<constraint>.+)[`"] FOREIGN KEY '
+    r'\([`"](?P<key>.+)[`"]\) REFERENCES [`"](?P<key_table>.+)[`"] ')
 def _foreign_key_error(integrity_error, match, engine_name, is_disconnect):
     """Filter for foreign key errors."""
 
@@ -250,8 +255,11 @@ def _check_constraint_error(
          "of relation "
          "\"(?P<relation>.+)\" does not exist")
 @filters("mysql", sqla_exc.InternalError,
-         r".*1091,.*Can't DROP '(?P<constraint>.+)'; "
-         "check that column/key exists")
+         r".*1091,.*Can't DROP (?:FOREIGN KEY )?['`](?P<constraint>.+)['`]; "
+         "check that .* exists")
+@filters("mysql", sqla_exc.OperationalError,
+         r".*1091,.*Can't DROP (?:FOREIGN KEY )?['`](?P<constraint>.+)['`]; "
+         "check that .* exists")
 @filters("mysql", sqla_exc.InternalError,
          r".*1025,.*Error on rename of '.+/(?P<relation>.+)' to ")
 def _check_constraint_non_existing(
@@ -277,6 +285,8 @@ def _check_constraint_non_existing(
          r".* no such table: (?P<table>.+)")
 @filters("mysql", sqla_exc.InternalError,
          r".*1051,.*Unknown table '(.+\.)?(?P<table>.+)'\"")
+@filters("mysql", sqla_exc.OperationalError,
+         r".*1051,.*Unknown table '(.+\.)?(?P<table>.+)'\"")
 @filters("postgresql", sqla_exc.ProgrammingError,
          r".* table \"(?P<table>.+)\" does not exist")
 def _check_table_non_existing(
@@ -285,22 +295,22 @@ def _check_table_non_existing(
     raise exception.DBNonExistentTable(match.group("table"), programming_error)
 
 
-@filters("ibm_db_sa", sqla_exc.IntegrityError, r"^.*SQL0803N.*$")
-def _db2_dupe_key_error(integrity_error, match, engine_name, is_disconnect):
-    """Filter for DB2 duplicate key errors.
+@filters("mysql", sqla_exc.InternalError,
+         r".*1049,.*Unknown database '(?P<database>.+)'\"")
+@filters("mysql", sqla_exc.OperationalError,
+         r".*1049,.*Unknown database '(?P<database>.+)'\"")
+@filters("postgresql", sqla_exc.OperationalError,
+         r".*database \"(?P<database>.+)\" does not exist")
+@filters("sqlite", sqla_exc.OperationalError,
+         ".*unable to open database file.*")
+def _check_database_non_existing(
+        error, match, engine_name, is_disconnect):
+    try:
+        database = match.group("database")
+    except IndexError:
+        database = None
 
-    N columns - (IntegrityError) SQL0803N  One or more values in the INSERT
-                statement, UPDATE statement, or foreign key update caused by a
-                DELETE statement are not valid because the primary key, unique
-                constraint or unique index identified by "2" constrains table
-                "NOVA.KEY_PAIRS" from having duplicate values for the index
-                key.
-
-    """
-
-    # NOTE(mriedem): The ibm_db_sa integrity error message doesn't provide the
-    # columns so we have to omit that from the DBDuplicateEntry error.
-    raise exception.DBDuplicateEntry([], integrity_error)
+    raise exception.DBNonExistentDatabase(database, error)
 
 
 @filters("mysql", sqla_exc.DBAPIError, r".*\b1146\b")
@@ -325,12 +335,22 @@ def _raise_mysql_table_doesnt_exist_asis(
          r"^.*1366.*Incorrect string value:*")
 @filters("sqlite", sqla_exc.ProgrammingError,
          r"(?i).*You must not use 8-bit bytestrings*")
+@filters("mysql", sqla_exc.DataError,
+         r".*1406.*Data too long for column.*")
 def _raise_data_error(error, match, engine_name, is_disconnect):
     """Raise DBDataError exception for different data errors."""
 
     raise exception.DBDataError(error)
 
 
+@filters("mysql", sqla_exc.OperationalError,
+         r".*\(1305,\s+\'SAVEPOINT\s+(.+)\s+does not exist\'\)")
+def _raise_savepoints_as_dberrors(error, match, engine_name, is_disconnect):
+    # NOTE(rpodolyaka): this is a special case of an OperationalError that used
+    # to be an InternalError. It's expected to be wrapped into oslo.db error.
+    raise exception.DBError(error)
+
+
 @filters("*", sqla_exc.OperationalError, r".*")
 def _raise_operational_errors_directly_filter(operational_error,
                                               match, engine_name,
@@ -351,14 +371,20 @@ def _raise_operational_errors_directly_filter(operational_error,
 
 
 @filters("mysql", sqla_exc.OperationalError, r".*\(.*(?:2002|2003|2006|2013|1047)")  # noqa
+@filters("mysql", sqla_exc.InternalError, r".*\(.*(?:1927)")  # noqa
+@filters("mysql", sqla_exc.InternalError, r".*Packet sequence number wrong")  # noqa
 @filters("postgresql", sqla_exc.OperationalError, r".*could not connect to server")  # noqa
-@filters("ibm_db_sa", sqla_exc.OperationalError, r".*(?:30081)")
 def _is_db_connection_error(operational_error, match, engine_name,
                             is_disconnect):
     """Detect the exception as indicating a recoverable error on connect."""
     raise exception.DBConnectionError(operational_error)
 
 
+@filters("*", sqla_exc.NotSupportedError, r".*")
+def _raise_for_NotSupportedError(error, match, engine_name, is_disconnect):
+    raise exception.DBNotSupportedError(error)
+
+
 @filters("*", sqla_exc.DBAPIError, r".*")
 def _raise_for_remaining_DBAPIError(error, match, engine_name, is_disconnect):
     """Filter for remaining DBAPIErrors.
@@ -369,8 +395,7 @@ def _raise_for_remaining_DBAPIError(error, match, engine_name, is_disconnect):
     if is_disconnect:
         raise exception.DBConnectionError(error)
     else:
-        LOG.exception(
-            _LE('DBAPIError exception wrapped from %s') % error)
+        LOG.warning('DBAPIError exception wrapped.', exc_info=True)
         raise exception.DBError(error)
 
 
@@ -381,9 +406,10 @@ def _raise_for_unicode_encode(error, match, engine_name, is_disconnect):
 
 @filters("*", Exception, r".*")
 def _raise_for_all_others(error, match, engine_name, is_disconnect):
-    LOG.exception(_LE('DB exception wrapped.'))
+    LOG.warning('DB exception wrapped.', exc_info=True)
     raise exception.DBError(error)
 
+
 ROLLBACK_CAUSE_KEY = 'oslo.db.sp_rollback_cause'
 
 
@@ -434,17 +460,21 @@ def handler(context):
                                 if isinstance(
                                         dbe, exception.DBConnectionError):
                                     context.is_disconnect = True
-                                raise
+                                return dbe
 
 
 def register_engine(engine):
-    event.listen(engine, "handle_error", handler)
+    event.listen(engine, "handle_error", handler, retval=True)
 
     @event.listens_for(engine, "rollback_savepoint")
     def rollback_savepoint(conn, name, context):
         exc_info = sys.exc_info()
         if exc_info[1]:
-            conn.info[ROLLBACK_CAUSE_KEY] = exc_info[1]
+            # NOTE(zzzeek) accessing conn.info on an invalidated
+            # connection causes it to reconnect, which we don't
+            # want to do inside a rollback handler
+            if not conn.invalidated:
+                conn.info[ROLLBACK_CAUSE_KEY] = exc_info[1]
         # NOTE(zzzeek) this eliminates a reference cycle between tracebacks
         # that would occur in Python 3 only, which has been shown to occur if
         # this function were in fact part of the traceback.  That's not the
@@ -456,7 +486,11 @@ def register_engine(engine):
     @event.listens_for(engine, "rollback")
     @event.listens_for(engine, "commit")
     def pop_exc_tx(conn):
-        conn.info.pop(ROLLBACK_CAUSE_KEY, None)
+        # NOTE(zzzeek) accessing conn.info on an invalidated
+        # connection causes it to reconnect, which we don't
+        # want to do inside a rollback handler
+        if not conn.invalidated:
+            conn.info.pop(ROLLBACK_CAUSE_KEY, None)
 
     # .. as well as connection pool checkin (just in case).
     # the .info dictionary lasts as long as the DBAPI connection itself
@@ -465,13 +499,3 @@ def register_engine(engine):
     @event.listens_for(engine, "checkin")
     def pop_exc_checkin(dbapi_conn, connection_record):
         connection_record.info.pop(ROLLBACK_CAUSE_KEY, None)
-
-
-def handle_connect_error(engine):
-    """Connect to the engine, including handle_error handlers.
-
-    The compat library now builds this into the engine.connect()
-    system as per SQLAlchemy 1.0's behavior.
-
-    """
-    return engine.connect()
