@@ -16,16 +16,15 @@ import copy
 import fixtures
 import pickle
 import sys
+from unittest import mock
 import warnings
 
-import mock
 from oslo_config import cfg
 from oslo_context import context as oslo_context
-from oslotest import base as oslo_test_base
 from sqlalchemy import Column
 from sqlalchemy import Integer
 from sqlalchemy import MetaData
-from sqlalchemy.orm import mapper
+from sqlalchemy.orm import registry
 from sqlalchemy.orm import Session
 from sqlalchemy import select
 from sqlalchemy import String
@@ -36,8 +35,9 @@ from oslo_db import options
 from oslo_db.sqlalchemy import enginefacade
 from oslo_db.sqlalchemy import engines as oslo_engines
 from oslo_db.sqlalchemy import orm
-from oslo_db.tests.sqlalchemy import base as test_base
-
+from oslo_db.tests import base as test_base
+from oslo_db.tests.sqlalchemy import base as db_test_base
+from oslo_db import warning
 
 enginefacade.transaction_context_provider(oslo_context.RequestContext)
 
@@ -91,7 +91,7 @@ class AssertDataSource(collections.namedtuple(
             assert False, "Unknown constant: %s" % const
 
 
-class MockFacadeTest(oslo_test_base.BaseTestCase):
+class MockFacadeTest(test_base.BaseTestCase):
     """test by applying mocks to internal call-points.
 
     This applies mocks to
@@ -357,11 +357,11 @@ class MockFacadeTest(oslo_test_base.BaseTestCase):
         maker_factories = mock.Mock(side_effect=get_maker)
 
         maker_factories(
-            autocommit=True, engine=engines.writer,
+            autocommit=False, engine=engines.writer,
             expire_on_commit=False)
         if self.slave_uri:
             maker_factories(
-                autocommit=True, engine=engines.async_reader,
+                autocommit=False, engine=engines.async_reader,
                 expire_on_commit=False)
 
         yield makers
@@ -479,6 +479,32 @@ class MockFacadeTest(oslo_test_base.BaseTestCase):
             [mock.call.dispose()]
         )
 
+    def test_started_flag(self):
+        facade = enginefacade.transaction_context()
+
+        self.assertFalse(facade.is_started)
+        facade.configure(connection=self.engine_uri)
+        facade.writer.get_engine()
+
+        self.assertTrue(facade.is_started)
+
+    def test_started_exception(self):
+        facade = enginefacade.transaction_context()
+
+        self.assertFalse(facade.is_started)
+        facade.configure(connection=self.engine_uri)
+        facade.writer.get_engine()
+
+        exc = self.assertRaises(
+            enginefacade.AlreadyStartedError,
+            facade.configure,
+            connection=self.engine_uri
+        )
+        self.assertEqual(
+            "this TransactionFactory is already started",
+            exc.args[0]
+        )
+
     def test_session_reader_decorator(self):
         context = oslo_context.RequestContext()
 
@@ -492,6 +518,19 @@ class MockFacadeTest(oslo_test_base.BaseTestCase):
                 with self._assert_reader_session(makers) as session:
                     session.execute("test")
 
+    def test_session_reader_decorator_kwarg_call(self):
+        context = oslo_context.RequestContext()
+
+        @enginefacade.reader
+        def go(context):
+            context.session.execute("test")
+        go(context=context)
+
+        with self._assert_engines() as engines:
+            with self._assert_makers(engines) as makers:
+                with self._assert_reader_session(makers) as session:
+                    session.execute("test")
+
     def test_connection_reader_decorator(self):
         context = oslo_context.RequestContext()
 
@@ -1146,7 +1185,7 @@ class MockFacadeTest(oslo_test_base.BaseTestCase):
                     session.execute("test")
 
 
-class PatchFactoryTest(oslo_test_base.BaseTestCase):
+class PatchFactoryTest(test_base.BaseTestCase):
 
     def test_patch_manager(self):
         normal_mgr = enginefacade.transaction_context()
@@ -1356,7 +1395,7 @@ class AsyncReaderWSlaveMockFacadeTest(MockFacadeTest):
     slave_uri = 'some_slave_connection'
 
 
-class LegacyIntegrationtest(test_base._DbTestCase):
+class LegacyIntegrationtest(db_test_base._DbTestCase):
 
     def test_legacy_integration(self):
         legacy_facade = enginefacade.get_legacy_facade()
@@ -1414,7 +1453,7 @@ class LegacyIntegrationtest(test_base._DbTestCase):
         )
 
 
-class ThreadingTest(test_base._DbTestCase):
+class ThreadingTest(db_test_base._DbTestCase):
     """Test copy/pickle on new threads using real connections and sessions."""
 
     def _assert_ctx_connection(self, context, connection):
@@ -1607,7 +1646,7 @@ class ThreadingTest(test_base._DbTestCase):
                 assert session is not session2
 
 
-class LiveFacadeTest(test_base._DbTestCase):
+class LiveFacadeTest(db_test_base._DbTestCase):
     """test using live SQL with test-provisioned databases.
 
     Several of these tests require that multiple transactions run
@@ -1632,11 +1671,13 @@ class LiveFacadeTest(test_base._DbTestCase):
         metadata.create_all(self.engine)
         self.addCleanup(metadata.drop_all, self.engine)
 
+        reg = registry()
+
         class User(object):
             def __init__(self, name):
                 self.name = name
 
-        mapper(User, user_table)
+        reg.map_imperatively(User, user_table)
         self.User = User
 
     def _assert_ctx_connection(self, context, connection):
@@ -1651,11 +1692,12 @@ class LiveFacadeTest(test_base._DbTestCase):
         with enginefacade.writer.using(context) as session:
             session.add(self.User(name="u1"))
 
-        session = self.sessionmaker(autocommit=True)
-        self.assertEqual(
-            "u1",
-            session.query(self.User.name).scalar()
-        )
+        session = self.sessionmaker(autocommit=False)
+        with session.begin():
+            self.assertEqual(
+                "u1",
+                session.query(self.User.name).scalar()
+            )
 
     def test_transaction_rollback(self):
         context = oslo_context.RequestContext()
@@ -1671,11 +1713,12 @@ class LiveFacadeTest(test_base._DbTestCase):
 
         self.assertRaises(MyException, go, context)
 
-        session = self.sessionmaker(autocommit=True)
-        self.assertEqual(
-            None,
-            session.query(self.User.name).scalar()
-        )
+        session = self.sessionmaker(autocommit=False)
+        with session.begin():
+            self.assertEqual(
+                None,
+                session.query(self.User.name).scalar()
+            )
 
     @mock.patch.object(Session, 'commit')
     @mock.patch.object(Session, 'rollback')
@@ -1742,11 +1785,12 @@ class LiveFacadeTest(test_base._DbTestCase):
                 s2.add(self.User(name="u1"))
                 s2.flush()
 
-        session = self.sessionmaker(autocommit=True)
-        self.assertEqual(
-            "u1",
-            session.query(self.User.name).scalar()
-        )
+        session = self.sessionmaker(autocommit=False)
+        with session.begin():
+            self.assertEqual(
+                "u1",
+                session.query(self.User.name).scalar()
+            )
 
     def test_context_deepcopy_on_connection(self):
         context = oslo_context.RequestContext()
@@ -1763,13 +1807,14 @@ class LiveFacadeTest(test_base._DbTestCase):
 
             self._assert_ctx_connection(ctx2, conn2)
 
-        session = self.sessionmaker(autocommit=True)
-        self.assertEqual(
-            "u1",
-            session.query(self.User.name).scalar()
-        )
+        session = self.sessionmaker(autocommit=False)
+        with session.begin():
+            self.assertEqual(
+                "u1",
+                session.query(self.User.name).scalar()
+            )
 
-    @test_base.backend_specific("postgresql", "mysql")
+    @db_test_base.backend_specific("postgresql", "mysql")
     def test_external_session_transaction(self):
         context = oslo_context.RequestContext()
         with enginefacade.writer.using(context) as session:
@@ -1799,14 +1844,14 @@ class LiveFacadeTest(test_base._DbTestCase):
             session.begin()
             session.add(self.User(name="u4"))
 
-        session = self.sessionmaker(autocommit=True)
-
+        session = self.sessionmaker(autocommit=False)
         # inner transaction + second part of "outer" transaction were committed
-        self.assertEqual(
-            [("u2",), ("u3",), ("u4", )],
-            session.query(
-                self.User.name).order_by(self.User.name).all()
-        )
+        with session.begin():
+            self.assertEqual(
+                [("u2",), ("u3",), ("u4", )],
+                session.query(
+                    self.User.name).order_by(self.User.name).all()
+            )
 
     def test_savepoint_transaction_decorator(self):
         context = oslo_context.RequestContext()
@@ -1839,14 +1884,14 @@ class LiveFacadeTest(test_base._DbTestCase):
 
         go1(context)
 
-        session = self.sessionmaker(autocommit=True)
-
+        session = self.sessionmaker(autocommit=False)
         # inner transaction + second part of "outer" transaction were committed
-        self.assertEqual(
-            [("u1",), ("u3",), ("u4", )],
-            session.query(
-                self.User.name).order_by(self.User.name).all()
-        )
+        with session.begin():
+            self.assertEqual(
+                [("u1",), ("u3",), ("u4", )],
+                session.query(
+                    self.User.name).order_by(self.User.name).all()
+            )
 
     def test_savepoint_transaction(self):
         context = oslo_context.RequestContext()
@@ -1867,16 +1912,16 @@ class LiveFacadeTest(test_base._DbTestCase):
 
             session.add(self.User(name="u4"))
 
-        session = self.sessionmaker(autocommit=True)
-
+        session = self.sessionmaker(autocommit=False)
         # inner transaction + second part of "outer" transaction were committed
-        self.assertEqual(
-            [("u1",), ("u3",), ("u4", )],
-            session.query(
-                self.User.name).order_by(self.User.name).all()
-        )
+        with session.begin():
+            self.assertEqual(
+                [("u1",), ("u3",), ("u4", )],
+                session.query(
+                    self.User.name).order_by(self.User.name).all()
+            )
 
-    @test_base.backend_specific("postgresql", "mysql")
+    @db_test_base.backend_specific("postgresql", "mysql")
     def test_external_session_transaction_decorator(self):
         context = oslo_context.RequestContext()
 
@@ -1915,16 +1960,16 @@ class LiveFacadeTest(test_base._DbTestCase):
 
         go1(context)
 
-        session = self.sessionmaker(autocommit=True)
-
+        session = self.sessionmaker(autocommit=False)
         # inner transaction + second part of "outer" transaction were committed
-        self.assertEqual(
-            [("u2",), ("u3",), ("u4", )],
-            session.query(
-                self.User.name).order_by(self.User.name).all()
-        )
+        with session.begin():
+            self.assertEqual(
+                [("u2",), ("u3",), ("u4", )],
+                session.query(
+                    self.User.name).order_by(self.User.name).all()
+            )
 
-    @test_base.backend_specific("postgresql", "mysql")
+    @db_test_base.backend_specific("postgresql", "mysql")
     def test_external_connection_transaction(self):
         context = oslo_context.RequestContext()
         with enginefacade.writer.connection.using(context) as connection:
@@ -1954,18 +1999,19 @@ class LiveFacadeTest(test_base._DbTestCase):
             # add more state on the "outer" transaction
             connection.execute(self.user_table.insert().values(name="u4"))
 
-        session = self.sessionmaker(autocommit=True)
-        self.assertEqual(
-            [("u2",), ("u3",), ("u4", )],
-            session.query(
-                self.User.name).order_by(self.User.name).all()
-        )
+        session = self.sessionmaker(autocommit=False)
+        with session.begin():
+            self.assertEqual(
+                [("u2",), ("u3",), ("u4", )],
+                session.query(
+                    self.User.name).order_by(self.User.name).all()
+            )
 
-    @test_base.backend_specific("postgresql", "mysql")
+    @db_test_base.backend_specific("postgresql", "mysql")
     def test_external_writer_in_reader(self):
         context = oslo_context.RequestContext()
         with enginefacade.reader.using(context) as session:
-            ping = session.scalar(select([1]))
+            ping = session.scalar(select(1))
             self.assertEqual(1, ping)
 
             # we're definitely a reader
@@ -1989,12 +2035,13 @@ class LiveFacadeTest(test_base._DbTestCase):
             user = session.query(self.User).first()
             self.assertEqual("u1_commit", user.name)
 
-        session = self.sessionmaker(autocommit=True)
-        self.assertEqual(
-            [("u1_commit",)],
-            session.query(
-                self.User.name).order_by(self.User.name).all()
-        )
+        session = self.sessionmaker(autocommit=False)
+        with session.begin():
+            self.assertEqual(
+                [("u1_commit",)],
+                session.query(
+                    self.User.name).order_by(self.User.name).all()
+            )
 
     def test_replace_scope(self):
         # "timeout" is an argument accepted by
@@ -2148,16 +2195,18 @@ class LiveFacadeTest(test_base._DbTestCase):
 
 
 class MySQLLiveFacadeTest(
-        test_base._MySQLOpportunisticTestCase, LiveFacadeTest):
+    db_test_base._MySQLOpportunisticTestCase, LiveFacadeTest,
+):
     pass
 
 
 class PGLiveFacadeTest(
-        test_base._PostgreSQLOpportunisticTestCase, LiveFacadeTest):
+    db_test_base._PostgreSQLOpportunisticTestCase, LiveFacadeTest,
+):
     pass
 
 
-class ConfigOptionsTest(oslo_test_base.BaseTestCase):
+class ConfigOptionsTest(test_base.BaseTestCase):
     def test_all_options(self):
         """test that everything in CONF.database.iteritems() is accepted.
 
@@ -2182,7 +2231,7 @@ class ConfigOptionsTest(oslo_test_base.BaseTestCase):
 
         self.assertEqual(1, len(w))
         self.assertTrue(
-            issubclass(w[-1].category, exception.NotSupportedWarning))
+            issubclass(w[-1].category, warning.NotSupportedWarning))
         self.assertEqual(
             "Configuration option(s) ['fake1', 'wrong2'] not supported",
             str(w[-1].message)
@@ -2202,7 +2251,7 @@ class ConfigOptionsTest(oslo_test_base.BaseTestCase):
         )
 
 
-class TestTransactionFactoryCallback(oslo_test_base.BaseTestCase):
+class TestTransactionFactoryCallback(test_base.BaseTestCase):
 
     def test_setup_for_connection_called_with_profiler(self):
         context_manager = enginefacade.transaction_context()
