@@ -10,15 +10,16 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from unittest import mock
+
 import alembic
-import mock
-from oslotest import base as test_base
 import sqlalchemy
 
 from oslo_db import exception
 from oslo_db.sqlalchemy.migration_cli import ext_alembic
 from oslo_db.sqlalchemy.migration_cli import ext_migrate
 from oslo_db.sqlalchemy.migration_cli import manager
+from oslo_db.tests import base as test_base
 
 
 class MockWithCmp(mock.MagicMock):
@@ -285,11 +286,11 @@ class TestMigrationMultipleExtensions(test_base.BaseTestCase):
 
     def test_upgrade_right_order(self):
         results = self.migration_manager.upgrade(None)
-        self.assertEqual(results, [100, 200])
+        self.assertEqual([100, 200], results)
 
     def test_downgrade_right_order(self):
         results = self.migration_manager.downgrade(None)
-        self.assertEqual(results, [100, 0])
+        self.assertEqual([100, 0], results)
 
     def test_upgrade_does_not_go_too_far(self):
         self.first_ext.obj.has_revision.return_value = True
